package com.yihu.ehr.resource.controller;

import com.fasterxml.jackson.core.type.TypeReference;
import com.yihu.ehr.constants.ApiVersion;
import com.yihu.ehr.constants.ServiceApi;
import com.yihu.ehr.controller.EnvelopRestEndPoint;
import com.yihu.ehr.model.resource.MRsReportView;
import com.yihu.ehr.resource.model.RsReportView;
import com.yihu.ehr.resource.service.RsReportViewService;
import com.yihu.ehr.util.rest.Envelop;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * 资源报表视图配置 服务接口
 *
 * @author 张进军
 * @created 2017.8.22 14:05
 */
@RestController
@RequestMapping(value = ApiVersion.Version1_0)
@Api(value = "RsReportViewEndPoint", description = "资源报表视图配置", tags = {"资源服务-资源报表视图配置"})
public class RsReportViewEndPoint extends EnvelopRestEndPoint {

    @Autowired
    private RsReportViewService rsReportViewService;

    @ApiOperation(value = "根据资源报表ID，获取资源报表视图配置")
    @RequestMapping(value = ServiceApi.Resources.RsReportViews, method = RequestMethod.GET)
    List<MRsReportView> findByReportId(
            @ApiParam(name = "reportId", value = "资源报表ID")
            @RequestParam(value = "reportId") Integer reportId) throws Exception {
        List<RsReportView> rsReportViews = rsReportViewService.findByReportId(reportId);
        return (List<MRsReportView>) convertToModels(rsReportViews, new ArrayList<MRsReportView>(), MRsReportView.class, "");
    }

    @ApiOperation("保存资源报表视图配置")
    @RequestMapping(value = ServiceApi.Resources.RsReportViewSave, method = RequestMethod.POST)
    public void save(
            @ApiParam(name = "reportId", value = "资源报表ID", required = true)
            @RequestParam(value = "reportId") Integer reportId,
            @ApiParam(name = "modelListJson", value = "资源报表视图配置集合JSON字符串", required = true)
            @RequestParam(value = "modelListJson") String modelListJson) throws Exception {
        List<RsReportView> rsReportViewList = objectMapper.readValue(modelListJson, new TypeReference<List<RsReportView>>(){});
        rsReportViewService.save(reportId, rsReportViewList);
    }

    @ApiOperation("判断资源报表视图配置是否存在")
    @RequestMapping(value = ServiceApi.Resources.RsReportViewExist, method = RequestMethod.GET)
    public boolean exist(
            @ApiParam(name = "reportId", value = "资源报表ID", required = true)
            @RequestParam(value = "reportId") Integer reportId,
            @ApiParam(name = "resourceId", value = "视图ID", required = true)
            @RequestParam(value = "resourceId") String resourceId) throws Exception {
        RsReportView rsReportView = rsReportViewService.findByReportIdAndResourceId(reportId, resourceId);
        if (rsReportView != null) {
            return true;
        }
        return false;
    }

    @ApiOperation("判断资源报表是否关联相关资源")
    @RequestMapping(value = ServiceApi.Resources.RsReportViewExistByResourceId, method = RequestMethod.GET)
    public Envelop existByResourceId(
            @ApiParam(name = "resourceId", value = "视图ID", required = true)
            @RequestParam(value = "resourceId") String resourceId){
        Envelop envelop = new Envelop();
        List<RsReportView> rsReportViewList = rsReportViewService.findByResourceId(resourceId);
        envelop.setSuccessFlg(true);
        if (rsReportViewList == null || rsReportViewList.size() <= 0) {
            envelop.setObj(false);
        }else {
            envelop.setObj(true);
        }
        return envelop;
    }

    @ApiOperation("判断资源视图是否存在于报表中")
    @RequestMapping(value = ServiceApi.Resources.RsReportViewExistReport, method = RequestMethod.GET)
    public boolean existReport(
            @ApiParam(name = "resourceId", value = "视图ID", required = true)
            @RequestParam(value = "resourceId") String resourceId) throws Exception {
        List<RsReportView> list = rsReportViewService.findByResourceId(resourceId);
        if (list != null && list.size() > 0) {
            return true;
        }
        return false;
    }

}
